/******************************************************************************
**+-------------------------------------------------------------------------+**
**|                            ****                                         |**
**|                            ****                                         |**
**|                            ******o***                                   |**
**|                      ********_///_****                                  |**
**|                      ***** /_//_/ ****                                  |**
**|                       ** ** (__/ ****                                   |**
**|                           *********                                     |**
**|                            ****                                         |**
**|                            ***                                          |**
**|                                                                         |**
**|         Copyright (c) 2008-2010 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                              |**
**|                                                                         |**
**| Permission is hereby granted to licensees of Texas Instruments          |**
**| Incorporated (TI) products to use this computer program for the sole    |**
**| purpose of implementing a licensee product based on TI products.        |**
**| No other rights to reproduce, use, or disseminate this computer         |**
**| program, whether in part or in whole, are granted.                      |**
**|                                                                         |**
**| TI makes no representation or warranties with respect to the            |**
**| performance of this computer program, and specifically disclaims        |**
**| any responsibility for any damages, special or consequential,           |**
**| connected with the use of this program.                                 |**
**|                                                                         |**
**+-------------------------------------------------------------------------+**
******************************************************************************/
/** \file     audio_sample.c
 *
 *  \brief    sample application for demostration of audio playing
 *
 *  This file contains the implementation of the sample appliation for the
 *  demostartion of audio playing through the audio interface layer.
 *
 *             (C) Copyright 2008, Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version
 *
 *      0.1  Created  [25/06/2008] created newly for da8xxx pre silicon platform.
 */

/* ========================================================================== */
/*                            INCLUDE FILES                                   */
/* ========================================================================== */

#include <stdio.h>
#include <xdc/std.h>
#include <ti/sdo/io/Stream.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/Log.h>
#include <ti/sdo/io/DriverTypes.h>
#include <xdc/runtime/Memory.h>
#include <ti/sysbios/heaps/HeapMem.h>
#include <ti/sysbios/BIOS.h>
#include <xdc/runtime/IHeap.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include "ti/psp/platforms/evmDA830/audio_evmInit.h"
#include "ti/psp/platforms/evmDA830/audio/Audio.h"
#include "ti/psp/mcasp/Mcasp.h"
/* ========================================================================== */
/*                          IMPORTED VARIABLES                                */
/* ========================================================================== */

extern Int edma3init();
extern EDMA3_DRV_Handle hEdma;
extern HeapMem_Handle myHeap;

/* ========================================================================== */
/*                          MACRO DEFINITIONS                                 */
/* ========================================================================== */

/*
 * Buffers placed in external memory are aligned on a 128 bytes boundary.
 * In addition, the buffer should be of a size multiple of 128 bytes for
 * the cache work optimally on the C6x.
 */
#define BUFLEN                  2560        /* number of samples in the frame */
#define BUFALIGN                128 /*alignment of buffer for use of L2 cache */


#define ONE_TO_ONE_TEST

/** Number of serializers configured for record */
#define RX_NUM_SERIALIZER       (1u)
#define TX_NUM_SERIALIZER       (1u)

#define BUFSIZE                 ((BUFLEN * sizeof(Ptr)))

#define NUM_BUFS                4 /* No of Buffers to be issued and reclaimed */

/* inStream and outStream are stream handles created in main */
static Stream_Handle inStream, outStream;

/* Function prototype */
static Void createStreams(Error_Block *eb);
static Void prime();

Mcasp_HwSetupData mcaspRcvSetup = {
        /* .rmask    = */ 0xFFFFFFFF, /* All the data bits are to be used     */
        /* .rfmt     = */ 0x0001CF70, /*
                                       * 0 bit delay from framsync
                                       * MSB first
                                       * No extra bit padding
                                       * Padding bit (ignore)
                                       * slot Size is 32 // TO CHECK LATER-VVIP
                                       * Reads from DMA port
                                       * NO rotation
                                       */
        /* .afsrctl  = */ 0x00000110, /* burst mode,
                                       * Frame sync is one bit
                                       * Rising edge is start of frame //TO CHECK LATER
                                       * externally generated frame sync
                                       */
        /* .rtdm     = */ 0x00000003, /* slot 1 is active (DSP)               */
        /* .rintctl  = */ 0x00000003, /* sync error and overrun error         */
        /* .rstat    = */ 0x000001ff, /* reset any existing status bits       */
        /* .revtctl  = */ 0x00000000, /* DMA request is enabled or disabled   */
        {
             /* .aclkrctl  = */ 0x00000000,
             /* .ahclkrctl = */ 0x00000000,
             /* .rclkchk   = */ 0x00000000
        }
} ;

Mcasp_HwSetupData mcaspXmtSetup = {
        /* .xmask    = */ 0xFFFFFFFF, /* All the data bits are to be used     */
        /* .xfmt     = */ 0x0001CF70, /*
                                       * 0 bit delay from framsync
                                       * MSB first
                                       * No extra bit padding
                                       * Padding bit (ignore)
                                       * slot Size is 32
                                       * Reads from DMA port
                                       * NO rotation
                                       */
        /* .afsxctl  = */ 0x00000110, /* burst mode,
                                       * Frame sync is one bit
                                       * Rising edge is start of frame
                                       * externally generated frame sync
                                       */
        /* .xtdm     = */ 0x00000003, /* slot 1 is active (DSP)               */
        /* .xintctl  = */ 0x00000007, /* sync error,overrun error,clK error   */
        /* .xstat    = */ 0x000001ff, /* reset any existing status bits       */
        /* .xevtctl  = */ 0x00000000, /* DMA request is enabled or disabled   */
        {
             /* .aclkxctl  = */ 0x00000000,
             /* .ahclkxctl = */ 0x00000000,
             /* .xclkchk   = */ 0x00000000
        },

};


/* McAsp channel parameters                                      */
Mcasp_ChanParams  mcasp_chanparam[Audio_NUM_CHANS]=
{
    {
        0x0001,                    /* number of serialisers      */
        {Mcasp_SerializerNum_0,}, /* serialiser index           */
        &mcaspRcvSetup,
		TRUE,
        Mcasp_OpMode_TDM,          /* Mode (TDM/DIT)             */
        Mcasp_WordLength_16,
        NULL,
        0,
        NULL,
        NULL,
        2,                        /* number of TDM channels      */
        ti_psp_mcasp_Mcasp_BufferFormat_1SER_MULTISLOT_INTERLEAVED,
        TRUE,
        TRUE
    },
    {
        0x0001,                   /* number of serialisers       */
        {Mcasp_SerializerNum_5,},
        &mcaspXmtSetup,
        TRUE,
        Mcasp_OpMode_TDM,
        Mcasp_WordLength_16,      /* word width                  */
        NULL,
        0,
        NULL,
        NULL,
        2,                        /* number of TDM channels      */
        ti_psp_mcasp_Mcasp_BufferFormat_1SER_MULTISLOT_INTERLEAVED,
        TRUE,
        TRUE
    }
};

Audio_ChannelConfig audioIfChanDefaultInfo[2]=
{
   /*  channel 0 (RX)                                             */
    &mcasp_chanparam[0],      /* channel params                   */
    {   /* codec [0]                                              */
        {
			48000,            /* sampling rate                    */		
            30,               /* gain                             */
            0x00,
            0x00
        }
    },
    /*  channel 1 (TX)                                            */
    &mcasp_chanparam[1],      /* channel params                   */
    {
        /* codec [0]                                              */
        {
            48000,            /* sampling rate                    */
            70,               /* gain                             */
            0x00,
            0x00
        }
    }
};

/*
 * ======== createStreams ========
 */
static Void createStreams(Error_Block *eb)
{
    Stream_Params  streamparams;
    Mcasp_ChanParams *chanparams = NULL;

    /* align the buffer to allow it to be used with L2 cache */
    Stream_Params_init(&streamparams);

    streamparams.maxIssues = NUM_BUFS;
    chanparams = audioIfChanDefaultInfo[1].chanParam;
    chanparams->edmaHandle = hEdma;
    streamparams.chanParams =(UArg)&audioIfChanDefaultInfo[DriverTypes_OUTPUT];
    /* open the I/O streams */
    outStream = Stream_create("/audio0", DriverTypes_OUTPUT,&streamparams,eb);

    if (outStream == NULL)
    {
        printf("Create output stream FAILED.%d",Error_getCode(eb));
        Error_print(eb);
        return;
    }

    chanparams = audioIfChanDefaultInfo[0].chanParam;
    chanparams->edmaHandle = hEdma;

    streamparams.chanParams =(UArg)&audioIfChanDefaultInfo[DriverTypes_INPUT];
    inStream = Stream_create("/audio0", DriverTypes_INPUT,&streamparams,eb);

    if (inStream == NULL)
    {
        printf("Create input stream FAILED.%d",Error_getCode(eb));
        return;
    }
}

Ptr buf[NUM_BUFS * 2];
/*
 * ======== prime ========
 */
static Void prime()
{
    Int32        count = 0;
    IHeap_Handle iheap;

    iheap = HeapMem_Handle_to_xdc_runtime_IHeap(myHeap);

    /* Allocate buffers for the stream buffer exchanges */
    for(count = 0; count < (NUM_BUFS ); count ++)
    {
        buf[count] = (Ptr)Memory_calloc(iheap, BUFSIZE * RX_NUM_SERIALIZER, BUFALIGN,NULL);
        if(NULL == buf[count])
        {
            printf("MEM_calloc failed.");
        }
    }

    /* Allocate buffers for the stream buffer exchanges */
    for(count = NUM_BUFS; count < (NUM_BUFS * 2); count ++)
    {
        buf[count] = (Ptr)Memory_calloc(iheap, BUFSIZE * TX_NUM_SERIALIZER, BUFALIGN,NULL);
        if(NULL == buf[count])
        {
            printf("MEM_calloc failed.");
        }
    }

    for(count = 0; count < NUM_BUFS; count ++)
    {
        /* Issue the first & second empty buffers to the input stream */
        Stream_issue(inStream, buf[count], BUFSIZE * RX_NUM_SERIALIZER,NULL, NULL);
    }

    for(count = NUM_BUFS; count < (NUM_BUFS * 2); count ++)
    {
        Stream_issue(outStream, buf[count], BUFSIZE * TX_NUM_SERIALIZER,NULL, NULL);
    }
}
static Int16 sinetable[48] = {
    0x0000, 0x10b4, 0x2120, 0x30fb, 0x3fff, 0x4dea, 0x5a81, 0x658b,
    0x6ed8, 0x763f, 0x7ba1, 0x7ee5, 0x7ffd, 0x7ee5, 0x7ba1, 0x763f,
    0x6ed8, 0x658b, 0x5a81, 0x4dea, 0x3fff, 0x30fb, 0x2120, 0x10b4,
    0x0000, 0xef4c, 0xdee0, 0xcf06, 0xc002, 0xb216, 0xa57f, 0x9a75,
    0x9128, 0x89c1, 0x845f, 0x811b, 0x8002, 0x811b, 0x845f, 0x89c1,
    0x9128, 0x9a76, 0xa57f, 0xb216, 0xc002, 0xcf06, 0xdee0, 0xef4c
};

/*
 * ======== echo ========
 *  This function copies from the input Stream to the output stream. You could
 *  easily replace the copy function with a signal processing algorithm.
 */
Void Audio_echo_Task(UArg a,UArg b)
{
    volatile int i32Count;
    Int nmadus = 0;
    Ptr rcv,xmt;
    Error_Block eb;
	int i;
	int sinetableIndex=0;

    /* configure the mcasp pinmux                                             */
    configureAudio();

    /* initialise the error block before it use                               */
    Error_init(&eb);

    /* initialise the edma library                                            */
    edma3init();

    /* Call createStream function to create I/O streams                       */
    createStreams(&eb);

    /* Call prime function to do priming                                      */
    prime();

    /* Forever loop to continously receviec and transmit audio data           */
    for (i32Count = 0; i32Count >= 0; i32Count++)
    {

        nmadus = Stream_reclaim(inStream,(Ptr *)&rcv,0xFFFFFFFF,NULL,NULL);

        /* Reclaim full buffer from the input stream */
        if (nmadus < 0)
        {
            printf("Error reclaiming full buffer from the input stream");
        }

        /* Reclaim empty buffer from the output stream to be reused */
        nmadus = Stream_reclaim(outStream, (Ptr *)&xmt,0xFFFFFFFF,NULL,NULL);
        if (nmadus < 0)
        {
            printf("Error reclaiming empty buffer from the output stream");
        }

        /* copy the receive information to the transmit buffer                */
        //memcpy(xmt,rcv,BUFSIZE * RX_NUM_SERIALIZER);

		unsigned short *xmtPtr=(unsigned short *)xmt;
		unsigned short *sinetablePtr= (unsigned short *)&sinetable[0];
		for( i=0;i<((BUFSIZE * RX_NUM_SERIALIZER)/4);i++)
		{
			if(sinetableIndex==48)
			sinetableIndex=0;
			*xmtPtr++=(unsigned short)sinetablePtr[sinetableIndex];
			*xmtPtr++=(unsigned short)sinetablePtr[sinetableIndex];
			sinetableIndex++;
		
		}
		
        /* Issue full buffer to the output stream                             */
        Stream_issue(outStream, xmt, BUFSIZE * TX_NUM_SERIALIZER,NULL,&eb);

        if (TRUE ==  Error_check(&eb))
        {
            printf("Failed to issue empty buffer to stream %s\n",Error_getMsg(&eb));
            Error_init(&eb);
        }

        /* Issue an empty buffer to the input stream                          */
        Stream_issue(inStream, rcv, BUFSIZE * RX_NUM_SERIALIZER,NULL,&eb);

        if (TRUE ==  Error_check(&eb))
        {
            printf("Failed to issue empty buffer to stream %s\n",Error_getMsg(&eb));
            Error_init(&eb);
        }
    }
}

/*============================================================================*/
/*                          END OF FILE                                       */
/*============================================================================*/
