/*******************************************************************************
**+--------------------------------------------------------------------------+**
**|                            ****                                          |**
**|                            ****                                          |**
**|                            ******o***                                    |**
**|                      ********_///_****                                   |**
**|                      ***** /_//_/ ****                                   |**
**|                       ** ** (__/ ****                                    |**
**|                           *********                                      |**
**|                            ****                                          |**
**|                            ***                                           |**
**|                                                                          |**
**|         Copyright (c) 1998-2006 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                               |**
**|                                                                          |**
**| Permission is hereby granted to licensees of Texas Instruments           |**
**| Incorporated (TI) products to use this computer program for the sole     |**
**| purpose of implementing a licensee product based on TI products.         |**
**| No other rights to reproduce, use, or disseminate this computer          |**
**| program, whether in part or in whole, are granted.                       |**
**|                                                                          |**
**| TI makes no representation or warranties with respect to the             |**
**| performance of this computer program, and specifically disclaims         |**
**| any responsibility for any damages, special or consequential,            |**
**| connected with the use of this program.                                  |**
**|                                                                          |**
**+--------------------------------------------------------------------------+**
*******************************************************************************/

/** \file   audioSample_main.c
 *
 *  \brief    sample application for demonstration of audio driver usage
 *
 *  This file contains the implementation of the sample appliation for the
 *  demonstration of audio playing through the audio interface layer.
 *
 *             (C) Copyright 2008, Texas Instruments, Inc
 *
 *  \author    Platform Support Group
 *
 *  \version
 *
 *      0.1  Created  [25/06/2008] created newly for da8xxx pre silicon platform.
 */

/* ========================================================================== */
/*                            INCLUDE FILES                                   */
/* ========================================================================== */

#include <stdio.h>
#include <xdc/std.h>
#include <xdc/runtime/Error.h>
#include <ti/sysbios/BIOS.h>
#include "ti/psp/psc/Psc.h"

/* ========================================================================== */
/*                          IMPORTED VARIABLES                                */
/* ========================================================================== */

extern Psc_Handle psc1;

/* ========================================================================== */
/*                           MACRO DEFINTIONS                                 */
/* ========================================================================== */

#define PSP_DEBUG  printf


/* mcasp1 module LPSC number      */
#define PSC_MCASP1_LPSC  8

/* ========================================================================== */
/*                           FUNCTION DEFINITIONS                             */
/* ========================================================================== */


/**
 *  \brief  Void main(Void)
 *
 *   Main function of the sample application. This function enables
 *   the mcasp instance in the power sleep controller and also
 *   enables the pinmux for the mcasp 1 instance.
 *
 *  \param  None
 *  \return None
 */
Void main(Void)
{
    PSP_DEBUG("\nTII PSP Audio Sample Main\n");

    BIOS_start();
    return;
}

/**
 *  \brief  Void Audio_Startup_InitFxn(Void)
 *
 *   Startup function for the audio module. it is used to initilaise the required
 *   modules for the audio application to run.
 *
 *  \param  None
 *  \return None
 */
Void Audio_Startup_InitFxn(Void)
{
    Error_Block eb;
    
    Error_init(&eb);
    
    /* power on the Mcasp 1 instance in the PSC  */
    Psc_ModuleClkCtrl(psc1, PSC_MCASP1_LPSC, Psc_moduleClkCtrl_ENABLE, &eb);
    
    return;
} 


/* ========================================================================== */
/*                                END OF FILE                                 */
/* ========================================================================== */
